# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from jmb.core.admin.options import ExtendibleModelAdmin, OrderedModelAdmin
from jmb.core.forms.models import AdminModelForm
from jmb.news.models import News, Category, NewsTranslation, CategoryTranslation
from image_cropping import ImageCroppingMixin
from forms import NewsAdminForm


class CategoryTranslationInline(admin.TabularInline):
    model = CategoryTranslation
    extra = 1
    fields = ('language', 'name')


class NewsTranslationInline(admin.StackedInline):
    model = NewsTranslation
    extra = 0
    prepopulated_fields = {"slug": ("title",)}
    fieldsets = (
        (None, {
            'fields': (
                'language', 'title', 'slug',
                'short_description', 'full_description'
                )
        }),
    )


class CategoryAdmin(ExtendibleModelAdmin, OrderedModelAdmin):
    list_display = ('get_status_icon', 'name', 'slug', 'get_edit_icon', 'get_delete_icon')
    list_display_links = ('name',)
    prepopulated_fields = {"slug": ("name",)}
    model = Category
    save_on_top = True
    inlines = (CategoryTranslationInline,)
    search_fields = ("status", "name",)
    date_hierarchy = "date_create"
    list_per_page = 20
    ordering = ("ordering", "name",)
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create', 'last_modifier', 'date_last_modify',
            ),
            'classes': (
                'collapse',
            )
        }),
        (_('Ordering'), {
            'fields': ('ordering',),
            'classes': ('collapse',),
        }),
        (_('general admin'), {
            'fields': ('status',),
        }),
        (None, {"fields": ("name", "slug")}),
    )


class NewsAdmin(ImageCroppingMixin, ExtendibleModelAdmin, OrderedModelAdmin):
    model = News
    save_on_top = True
    filter_horizontal = ('users', 'categories',)
    inlines = (NewsTranslationInline,)
    list_display = (
        'get_status_icon', 'title', 'event_date',
        'get_categories', 'get_users',
        'get_edit_icon', 'get_delete_icon',
    )
    list_display_links = ('title',)
    form = NewsAdminForm
    list_per_page = 30
    ordering = ("ordering", "title",)
    ordering = ("categories",)
    prepopulated_fields = {"slug": ("title",)}
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('Ordering'), {
            'fields': ('ordering',),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': ('status', 'categories',),
        }),
        (None, {
            "fields": (
                'title', 'slug',
                'event_date', 'show_date_in_detail_page',
                'big_image', ('cropping_small', 'cropping_big'),
                'short_description', 'full_description',
                'link', 'file', 'note',
            ),
        }),
        (_('Publication date'), {
            'fields': ('pub_date_begin', 'pub_date_end',),
            'classes': ('collapse',),
        }),
        (_("SEO settings"), {
            'fields': ('page_title', 'meta_description','meta_keywords'),
            'classes': ('collapse',)
        }),
        # (_('Show to'), {
        #     'fields': ('users',),
        # }),
    )
    advanced_search_fields = (
        ('title__icontains',),
        ('categories__in',),
    )


admin.site.register(Category, CategoryAdmin)
admin.site.register(News, NewsAdmin)
