# -*- coding: utf-8 -*-

from cms.models.pluginmodel import CMSPlugin
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _

from jmb.core.db.models import StatusModel
from jmb.core.db.models import DateModel, UserModel, OrderedModel
from jmb.news.models import Category


class NewsSettings(CMSPlugin):
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
        blank=True, null=True,
    )
    category_slugs_included = models.CharField(
        _("Category slug"),
        max_length=100,
        blank=True,
        help_text=_("(Optional)News category slug(s) filter")
    )
    slugs_included = models.CharField(
        _("Slug"),
        max_length=100,
        blank=True,
        help_text=_("(Optional)News slug(s) filter")
    )
    category_slugs_excluded = models.CharField(
        _("Excluding category slugs"),
        max_length=100, blank=True,
        help_text=_("(Optional)excluding News category slug(s) filter")
    )
    slugs_excluded = models.CharField(
        _("Excluding slugs"),
        max_length=100, blank=True,
        help_text=_("(Optional)excluding News slug(s) filter")
    )
    pub_date_begin = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date begin'),
        help_text=_('News publication start date'),
    )
    pub_date_end = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date end'),
        help_text=_('When news publication date ends.'),
    )
    objects_per_page = models.IntegerField(
        default=10
    )
    adjacent_pages = models.IntegerField(
        default=2,
    )
    ordering = models.CharField(
        max_length=100,
        default='ordering, event_date, title'
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.NEWS_CMSPLUGIN_ALL_LIST_TEMPLATES,
        default="news/cms_plugins/news_box_wide.html"
    )
    date_group = models.CharField(
        max_length=100,
        null=True, blank=True,
        choices=settings.NEWS_CMSPLUGIN_DATE_GROUP_CHOICES,
        help_text=_("(Optional) if populated, output will be a list of dates on which event_date occurs")
    )
    news_page_slug = models.SlugField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("News page slug"),
        help_text=_("(Optional) News page slug")
    )

    class Meta:
        app_label = "news"

    def __unicode__(self):
        if self.name:
            return u"{0}".format(self.name)
        return u"{0}".format(self.template_name)


class NewsSettingCategory(DateModel, UserModel, OrderedModel):
    news_setting = models.ForeignKey(
        NewsSettings,
        verbose_name=_("News setting"),
        related_name='categories',
    )
    category = models.ForeignKey(
        Category,
        verbose_name=_('Category'),
        related_name='cmsplugin_news_setting_categories',
    )

    class Meta:
        app_label = "news"
        verbose_name = _("News category")
        verbose_name_plural = _("News categories")
        ordering = ('news_setting', 'ordering')

    def __unicode__(self):
        return u"{0}".format(self.news_setting.template_name)


class NewsSearchSettings(CMSPlugin, StatusModel):
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.NEWS_FORM_CLASSES,
        default="jmb.news.forms.SearchNewsForm",
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.NEWS_CMSPLUGIN_SEARCH_TEMPLATES,
        default="news/cms_plugins/search.html"
    )

    class Meta:
        app_label = "news"

    def __unicode__(self):
        return u"{0}".format(self.template_name)


class CategorySettings(CMSPlugin):
    slugs_included = models.CharField(
        _("Included Slug"),
        max_length=100,
        blank=True,
        help_text=_("(Optional)Categories slug(s) filter")
    )
    slugs_excluded = models.CharField(
        _("Excluding slugs"),
        max_length=100, blank=True,
        help_text=_("(Optional)excluding Categories slug(s) filter")
    )
    objects_per_page = models.IntegerField(default=10)
    ordering = models.CharField(max_length=100, default='ordering, name')
    template_name = models.CharField(
        max_length=100,
        choices=settings.NEWS_CATEGORY_CMSPLUGIN_LIST_TEMPLATES,
        default="news/cms_plugins/category_list.html"
    )
    news_page_slug = models.SlugField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("News page slug"),
        help_text=_("(Optional) News page slug")
    )

    class Meta:
        app_label = "news"

    def __unicode__(self):
        return u"{0}".format(self.template_name)


class NewsCategoryFilterSettings(CMSPlugin):
    news_plugin_setting = models.OneToOneField(
        NewsSettings,
        related_name="category_filter_settings"
    )
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.NEWS_CATEGORY_FILTER_CMSPLUGIN_TEMPLATES,
        default="bootstrap3/news/cms_plugins/news_category_filter.html",
    )

    def __unicode__(self):
        return u"{0}".format(self.template_name)

    class Meta:
        app_label = "news"


class NewsCategorySelectFilterSettings(CMSPlugin):
    news_plugin_setting = models.OneToOneField(
        NewsSettings,
        related_name="category_select_filter_settings"
    )
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.NEWS_CATEGORY_SELECT_FILTER_CMSPLUGIN_TEMPLATES,
        default="bootstrap3/news/cms_plugins/news_category_select_filter.html",
    )

    def __unicode__(self):
        return u"{0}".format(self.template_name)

    class Meta:
        app_label = "news"
